var RotatingTimer = Class.extend({

	list: null,
	t: 0.0,
	i: 0,
	
	construct: function(list)
	{
		this.list = list;
		this.t = 0.0;
		this.i = 0;
	},
	
	update: function ()
	{
		this.t += (1.0 / this.list[this.i][1]) / FPS;
		if (this.t >= 1)
		{
			this.t -= 1;
			this.i = (this.i + 1) % this.list.length;
		}
		return this.list[this.i][0];
	}

});

var DelayedTracker = Class.extend({

	list: null,
	obj: null,
	
	construct: function (obj)
	{
		this.list = [];
		this.obj = obj;
	},
	
	track: function (obj, dtime)
	{
		if (this.obj === null || obj.id !== this.obj.id)
		{
			this.obj = obj;
			this.list = [];
			return null;
		}
		
		if (obj === null)
			return null;
		
		this.list.push(obj.getPosition());
		
		if (this.list.length > dtime*FPS)
		{
			var ret = this.list[0];
			this.list.splice(0, 1);
			return ret;
		}
		
		return null;
	},

});

var DistanceDelayedTracker = Class.extend({

	dt: null,
	
	construct: function (obj)
	{
		this.dt = new DelayedTracker(obj);
	},
	
	track: function (obj, me, dtime1, dtime2, dist1, dist2)
	{
		if (obj === null)
			return this.dt.track(obj);
		
		var dist = obj.getPosition().distance(me.getPosition());
		
		if (dist <= dist1)
			return this.dt.track(obj, dtime1);
		else if (dist >= dist2)
			return this.dt.track(obj, dtime2);
		else
		{
			var dtime = (dist-dist1)/(dist2-dist1) * (dtime2-dtime1) + dtime1;
			return this.dt.track(obj, dtime);
		}
	}

});

var Lazers = Class.extend({
	list: null,
	tlife: 0.1,
	tick: 0,
	
	construct: function()
	{
		this.list = [];
	},
	
	updateRender: function()
	{
		this.tick += 1;
		for (var i=0; i<this.list.length; i++)
		{
			var t = (ctime() - this.list[i].st) / this.tlife;
			if (t >= 1.0)
			{
				this.list.splice(i, 1);
				i -= 1;
				continue;
			}
			var c = getCam();
			var ps = this.list[i].obj.getPosition();
			var pe = this.list[i].to.clone();
			var ar = this.list[i].obj.attackRange?this.list[i].obj.attackRange/c.z:(XRES/2.0)/c.z;
			var dir = pe.sub(ps);
			if (dir.length() > ar)
				pe = ps.add(dir.norm().mulf(ar));
			
			var rayc = fireRay(ps, pe, this.list[i].obj.cgroup);
			if (rayc)
			{
				pe = rayc.hit.clone();
				var sp = (Math.random()+0.5)*64.0;
				if (this.tick%2)
					addParticle(pe, new pnt(Math.random()*rayc.norm.x*sp, Math.random()*rayc.norm.y*sp), 2.0, rgb(1.0, sp/128.0, 0.0), 0.25);
				if (rayc.obj)
				{
					var im = rayc.norm.mulf(-0.25);
					rayc.obj.addVelocity(im.mulf(1.0/c.z));
					if (rayc.obj.damage)
						rayc.obj.damage(im.length()/5.0*(this.list[i].obj.attackPower?this.list[i].obj.attackPower:1.0));
				}
			}
			
			var sps = c.toScr(ps);
			//drawCircleFG(sps.x, sps.y, 0.0, rgba(0,0,0,0), this.list[i].obj.getRadius(), rgba(0, 0, 1.0, (1.0-t)));
			var dir = ps.sub(pe);
			var na = Math.random() * Math.PI + Math.PI/2.0 + Math.atan2(dir.y, dir.x);
			var n = new pnt(Math.cos(na), Math.sin(na));
			ps = ps.add(pe.sub(ps).norm().mulf(this.list[i].obj.getRadius()/c.z/16.0));
			ps = ps.add(n.mulf(this.list[i].obj.getRadius()/c.z/8.0));
			sps = c.toScr(ps);
			spe = sps.add(pe.sub(ps).norm().mulf(pe.sub(ps).length()*t));
			var spe = c.toScr(pe);
			
			if (this.list[i].obj.cgroup === 1)
				drawLine(sps.x, sps.y, spe.x, spe.y, rgba(0.0, 1.0, 0.5, 1.0-t));
			else
				drawLine(sps.x, sps.y, spe.x, spe.y, rgba(1.0, 0.0, 0.0, 1.0-t));
		}
	},
	
	add: function(obj, to, range)
	{
		this.list.push({
			st: ctime(),
			obj: obj,
			to: to,
			range: range
		});
	}
});

var Stars = Class.extend({
	clr: null,
	z: null,
	size: null,
	stars: null,
	bfr: 30,
	lastCam: null,
	
	construct: function(n_stars, z, size, clr)
	{
		this.clr = clr;
		this.size = size;
		this.z = z;
		this.lastCam = getCam();
		this.stars = [];
		
		for (var i=0; i<n_stars; i++)
		{
			var ns = { p: null };
			ns.p = new pnt(Math.random()*(XRES+this.bfr*2)-this.bfr, Math.random()*(YRES+this.bfr*2)-this.bfr);
			this.stars.push(ns);
		}
	},
	
	updateRender: function()
	{
		var tcam = getCam();
		var offset = tcam.p.sub(this.lastCam.p).mulf(1.0/this.z);
		
		var min = new pnt(-this.bfr, -this.bfr), max = new pnt(XRES+this.bfr, YRES+this.bfr);
		
		for (var i=0; i<this.stars.length; i++)
		{
			var s = this.stars[i];
			s.p = s.p.sub(offset);
			if (s.p.x < min.x || s.p.x > max.x)	s.p.y = Math.random()*(YRES+this.bfr*2)-this.bfr;
			if (s.p.y < min.y || s.p.y > max.y) s.p.x = Math.random()*(XRES+this.bfr*2)-this.bfr;
			while (s.p.x < min.x) s.p.x += max.x - min.x;
			while (s.p.x > max.x) s.p.x -= max.x - min.x;
			while (s.p.y < min.y) s.p.y += max.y - min.y;
			while (s.p.y > max.y) s.p.y -= max.y - min.y;
			drawCircleFG(s.p.x, s.p.y, 0.0, this.clr, this.size*2.0, rgba(0,0,0,0));
		}
		
		this.lastCam = tcam;		
	}
});

var CutsceneSequencer = Class.extend({
	list: null,
	ticker: 0.0,
	alpha: 1.0,
	retflag: null,
	ltime: null,
	
	construct: function()
	{
		this.clear();
	},
	
	clear: function ()
	{
		this.list = [];
		this.ticker = 0.0;
		this.alpha = 0.0;
		this.ltime = ctime();
	},
		
	complete: function ()
	{
		return this.list.length === 0;
	},
	
	setFlag: function (flag)
	{
		this.retflag = flag;
	},
	
	getFlag: function ()
	{
		return this.retflag;
	},
	
	gotoLoadPoint: function ( name )
	{
		while(this.list.length > 0 && this.list[0].flags.point !== name)
			this.list.splice(0, 1);
		if (this.list[0].flags.endCall) this.list[0].flags.endCall(this);
		for (var i=0; i<this.list.length; i++)
		{
			if (this.list[i].flags.point)
			{
				this.list.splice(i,1);
				i--;
			}
		}
	},
	
	addLoadPoint: function ( name, cbk )
	{
		this.addText('', 0.0, 0.0, { point:name, endCall:cbk });
	},
	
	addText: function ( text, time, outTime, flags )
	{
		this.list.push({
			text: 		text,
			time: 		time,
			outTime:	outTime,
			flags:      flags?flags:{},
		});
	},
	
	addImage: function ( text, img, imgSize, time, outTime )
	{
		this.addText(text, time, outTime, { img: img, imgSize: imgSize });
	},
	
	addWait: function ( text, cbk )
	{
		this.addText(text, 0.0, 0.0, { until:cbk });
	},
	
	addCall: function ( cbk )
	{
		this.addText("", 0.0, 0.0, { endCall:cbk });
	},
	
	addPause: function ( time )
	{
		this.addText("", time, 0.0);
	},
	
	updateRender: function ()
	{
		var ntime = ctime();
		var dt = ntime-this.ltime;
		if (dt*FPS > 2.0) dt = 2.0/FPS;
		this.ltime = ntime;
		if (this.list.length > 0 && this.list[0].point)
			this.list.splice(0,1);
		if (this.list.length > 0)
		{
			var fobar = false;
			if (this.list.length >= 2)
			{
				if (/*this.list[1].flags.until &&*/ !this.list[1].flags.endCall && this.list[1].text === '')
					fobar = true;
			}
			var nsbar = false;
			if (this.list.length >= 1)
				if (/*this.list[0].flags.until &&*/ !this.list[0].flags.endCall && this.list[0].text === '')
					nsbar = true;
			var bal = 1.0;
			if (this.list.length===1 || fobar)
				bal = this.alpha;
			if (nsbar)
				bal = 0.0;
			drawBoxF(0.0, YRES-48-15, XRES, YRES, rgba(0,0,0,bal));
			var th = this.list[0];
			if (this.ticker >= th.time && !(th.flags.until && th.flags.until(this.ticker, this) === false))
			{
				if (th.flags.endCall)
					th.flags.endCall(this);
				this.ticker = 0.0;
				this.list.splice(0, 1);
				this.alpha = 1.0;
			}
			else
			{
				this.ticker += dt;
				if (this.ticker >= (th.time-th.outTime))
				{
					this.alpha -= dt*(1.0/th.outTime);
					if (this.alpha < 0.0) this.alpha = 0.0;
				}
				var flash = 0;
				if (th.flags.flashing === true)
				{
					flash = Math.sin(this.ticker * 3.0 * Math.PI) * 0.5 + 0.5;
					flash = (flash*0.4);
				}
				if (th.flags.until) this.alpha = 1.0;
				var pos = Math.min(Math.floor(this.ticker*64.0), th.text.length);
				if (pos < th.text.length) flash = 0;
				var r = 1, g = 1, b = 1;
				if (th.flags.color === "green")
				{
					r = 0; g = 1; b = 0;
				}
				else if (th.flags.color === "red")
				{
					r = 1; g = 0; b = 0;
				}
				drawString(XRES/2-stringWidth(th.text, 16.0)/2.0, YRES-32-4-7.5-4, th.text.substring(0, pos), rgba(r,g,b, this.alpha*(1-flash)), 16, rgba(r/2,g/2,b/2, this.alpha*(1-flash)));
				if (th.flags.img)
				{
					drawImage(th.flags.img, XRES/2-th.flags.imgSize/2.0, YRES/2-th.flags.imgSize/2.0, th.flags.imgSize, this.alpha);
				}
			}
		}
	},
});

function explosion ( p, r, clrs, isBoss )
{
	var z = getCam().z;
	var n = Math.floor(r * 5);
	for (var i=0; i<n; i++)
	{
		var off = pntFromAngle(Math.random()*2.0*Math.PI).mulf(Math.random()*r);
		var pp = p.add(off);
		var rnd = Math.floor(Math.random()*(clrs.length-0.0001));
		var sz = Math.random()*3.0 + 3.0;
		if (isBoss)
			sz *= 6.0;
		addParticle(pp, off.mulf(15.0/(sz-2.0)*(isBoss?6.0:1.0)*z), sz, clrs[rnd], 0.5);
	}
}

var sp_tick = 0;
function sunParks ()
{
	for (var i=0; i<3; i++)
	{
		var p = getCam().fromScr(new pnt(Math.random()*(XRES+50)-25, Math.random()*(YRES+50)-25));
		var r = Math.random();
		var v = p.mulf(1.0/3.0);
		v = v.norm().mulf(500000.0/v.length());
		var sp = p.length() - SUN_RADIUS*0.65;
		sp = Math.pow(sp, 1.0/3.0);
		if (sp < 3.0) sp = 3.0;
		if (sp < 10.0)
		{
			addParticle(p, v, 10.0/sp, rgba(1.0, r, r, 1.0/sp), 1.0/sp);
		}
	}
}

var getObjPath = function (next)
{
	if (next.length > 0 && next.charAt(0) === '/')
		return next;
	else
	{
		var t = curObjective.split("/");
		t[t.length-1] = next;
		return t.join("/");
	}
};
var getObjStruct = function (path)
{
	var ob = getObjPath(path);
	var t = ob.split("/");
	var n = objective[t[1]];
	for (var i=2; i<t.length; i++)
		n = n.sub[t[i]];
	return n;
};
var getObjParentPath = function(path)
{
	var pt = getObjPath(path);
	var t = pt.split("/");
	t.splice(t.length-1, 1);
	return t.join("/");
};

var nextObjective = function (_cur)
{
	var cur = _cur?getObjStruct(_cur):getObjStruct(curObjective);
	if (cur.last === true)
	{
		curObjective = getObjParentPath(_cur?getObjStruct(_cur):getObjStruct(curObjective))
		cur = getObjStruct(curObjective);
		if (cur.next)
		{
			curObjective = getObjPath(cur.next);
			cur = getObjStruct(curObjective);
		}
		if (cur.first)
			curObjective += "/" + cur.first;
	}
	else if (cur.next)
		curObjective = getObjPath(cur.next);
};

var objectiveCompare = function(a, b)
{
	if (a === b)
		return 0;
	
	var ret = 1;
	
	var oco = curObjective;
	curObjective = a;
	while (ret === 1)
	{
		var old = curObjective;
		nextObjective();
		if (old === curObjective)
			break;
		if (curObjective === b)
			ret = -1;
	}
	curObjective = oco;
	
	return ret;
};

var pyramid1 = null;

var curObjective = '/MISSION1/INTRO';
var objective = {
	'MISSION1': {
		first: 'INTRO',
		sub: {
			'INTRO': {
				next: 'LEVELUP',
			},
			'LEVELUP': {
				next: 'BOSS',
			},
			'BOSS': {
				last: true,
			},
		},
		next: 'MISSION2',
	},
	'MISSION2': {
		first: 'TRAVEL',
		sub: {
			'TRAVEL': {
				last: true,
			},
		},
	}
};

var levelUps = {
	3: { money: 30, armour: 0.0, attackPower: 0.15, maxHealth: 0.25 },
	4: { money: 80, armour: 0.15, attackPower: 0.35, maxHealth: 0.15 },
	5: { money: 200, armour: 0.35, attackPower: 0.15, maxHealth: 0.15 }
};

var canMove = false, canShoot = true;

var CS = new CutsceneSequencer();
var CSSub = new CutsceneSequencer();
var CSLup = new CutsceneSequencer();
var loadTime = null;
var ship = getObjByClass("PShip1")[0];
var lazers = new Lazers();
var stars = [
	new Stars(3, 11.0, 5.0, rgba(1,1,1,0.5)),
	new Stars(7, 7.0, 3.0, rgba(.5,.5,1,0.5)),
	new Stars(7, 7.0, 3.0, rgba(1,.5,.5,0.5)),
	new Stars(15, 3.0, 2.0, rgba(.9,.9,.9,0.75)),
	new Stars(15, 1.0, 1.0, rgba(.6,.6,.6,0.75))
];
var shooting = false;
var lastmp;
var map_showing = 0.0;
var clrLevel = 4, curClrLevel = 0;
var playerMoney = null;
var playerLevel = 1;

var battles = {
	'intro': [
		{ enemies: [ ADrone ], weight: 1.0 },
	],
	'intro_outer': [
		{ enemies: [ ADrone, ADrone ], weight: 1.0 },
	],
	'home': [
		{ enemies: [ ADrone, ADrone ], weight: 0.4 },
		{ enemies: [ Ufo1 ], weight: 0.4 },
		{ enemies: [ ADrone, ADrone, ADrone ], weight: 0.2 },
	],
	'world2': [
		{ enemies: [ Ufo1, Ufo1 ], weight: 0.35 },
		{ enemies: [ ADrone, ADrone, ADrone ], weight: 0.65 },
	],
};

var lastSave = -10.0;
var lastLoad = -10.0;

var save = function (){
	var state = {};
	state.player = {
		money: playerMoney,
		level: playerLevel,
		position: {x: ship.getPosition().x, y: ship.getPosition().y},
		health: ship.health,
		maxHealth: ship.maxHealth,
		armour: ship.armour,
		attackRange: ship.attackRange,
		attackPower: ship.attackPower,
	}
	
	state.cameraZoom = getCam().z;
	
	var activeZones = [];
	for (var id in __zone)
		if (__zone[id].active === 1)
			activeZones.push(id);
	state.activeZones = activeZones;
		
	state.canMove = canMove;
	state.canShoot = canShoot;
	state.curObjective = curObjective;
	
	var cells = getObjByClass("Cell");
	var oCells = [];
	for (var i=0; i<cells.length; i++)
		oCells.push({x:cells[i].p.x, y:cells[i].p.y, worth:cells[i].worth});
	state.cells = oCells;
	
	setConfig('save_game_1', serialize(state));
	lastSave = ctime();
};

var load = function (){
	var str = getConfig('save_game_1');
	if (str && str !== "")
	{
		var state = eval('('+str+')');
		playerMoney = state.player.money;
		playerLevel = state.player.level;
		
		state.player.position = new pnt(state.player.position.x, state.player.position.y);
		ship.setPosition(state.player.position);
		ship.setVelocity(new pnt(0,0));
		var c = getCam();
		c.p = state.player.position.clone();
		c.z = state.cameraZoom;
		setCam(c);
		
		ship.maxHealth = state.player.maxHealth;
		ship.health = ship.maxHealth;
		ship.armour = state.player.armour;
		ship.attackRange = state.player.attackRange;
		ship.attackPower = state.player.attackPower;
		
		for (var i=0; i<state.cells.length; i++)
		{
			var cell = new Cell(new pnt(state.cells[i].x, state.cells[i].y));
			cell.worth = state.cells[i].worth;
			cell.updateRadius();
			addObject(cell);
		}
		
		curObjective = state.curObjective;
		
		for (var i=0; i<state.activeZones.length; i++)
		{
			zoneActive([__zone[parseInt(state.activeZones[i])]], 1);
		}
		
		canMove = state.canMove;
		canShoot = state.canShoot;
		lastLoad = ctime();	
		return true;
	}
	lastLoad = ctime();	
	return false;
};

var calcMiss = function ( distance, min_dist, max_dist, min_miss, max_miss )
{
	var t = (distance-min_dist) / (max_dist-min_dist);
	if (t < 0.0) t = 0.0;
	else if (t > 1.0) t = 1.0;
	var miss = t * (max_miss-min_miss) + min_miss;
	return pntFromAngle(Math.random()*Math.PI*2.0).mulf(miss*Math.random());
};

var fireProjectile = function(obj, pclass, target, _opts)
{
	var proj = eval("(new " + pclass + "(obj.getPosition()))");
	var opts = _opts?_opts:{};
	proj.cgroup = obj.cgroup;
	if (opts.powerf)
		proj.mass *= opts.powerf;
	proj.fireTowards(target, obj.getVelocity(), opts);
	addObject(proj);
}

var makeSafe = function(){
	if (in_battle === true)
	{
		var obs = dynamicObjectList();
		for (var i=0; i<obs.length; i++)
		{
			if (obs[i].cgroup === 2 && obs[i].className !== 'Cell')
				removeObject(obs[i]);
		}
	}
};

var objectives = [], tips = [];

var init = function (){
	if (TEST_POINT)
		ship.setPosition(TEST_POINT);
	var c = getCam();
	c.p = ship.getPosition();
	setCam(c);
			
	// Preload Images
	loadImage("game/graphics/ufo.png");
	loadImage("game/graphics/ufo2.png");
	loadImage("game/graphics/ship.png");
	loadImage("game/graphics/warp.png");
	loadImage("game/graphics/greenarrow.png");
	loadImage("game/graphics/hud-center.png");
	loadImage("game/graphics/pyramid.png");
	loadImage("game/graphics/pyramid-heal.png");
	loadImage("game/graphics/healorb.png");
	
	pyramid1 = closestObjectByClass(zonesByTag("obj1")[0].p, 'Pyramid');
	
	//loadTime = ctime(); canMove = true;
	playerMoney = 0;
		
	loadTime = ctime();
	if (!load())
	{
		curObjective = "/MISSION1/INTRO";
		zoneActive(zonesByTag("obj1"), 1);
		zoneActive(zonesByTag("intro"), 1);
		zoneActive(zonesByTag("safe", zonesByTag("home")), 1);
		zoneActive(zonesByTag("home"), 0);
	}

	CS.addLoadPoint("/MISSION1/INTRO", function() {
		loadTime = ctime() + 2.5 + 3.0 + 15.0 + 40.0 + 3.0;
	});
	CS.addPause(3.0);
	CS.addImage("", loadImage("game/graphics/intro-2.png"), YRES*0.8, 40.0, 1.0);
	CS.addImage("", loadImage("game/graphics/intro-1.png"), YRES*0.8, 15.0, 1.0);
	CS.addLoadPoint("_skip_intro", function() {
		loadTime = ctime() + 3.0;
	});
	CS.addPause(3.0);
	CS.addText("System Initialization...", 4.0, 1.5);
	CS.addText("ERROR: Visualization malfunction", 4.0, 0.25, {flashing: true, color: 'red'});
	CS.addText("ERROR: Data storage corrupted", 4.0, 0.25, {flashing: true, color: 'red'});
	CS.addText("STATUS: Hull strength at " + Math.floor(ship.health/ship.maxHealth*100) + "%", 4.0, 1.5);
	CS.addText("STATUS: Cannon active.", 4.0, 1.5);
	CS.addText("LOCATION: Unknown planet", 4.0, 1.5);
	CS.addCall(function(){canMove=true;
		objectives = [ "OBJECTIVE: Search planet for means to repair" ];
		tips = [
			"TIP: Arrow keys or WASD to move",
			"TIP: Aim with the mouse and left click to shoot"
		];
	});
	CS.addText("OBJECTIVE: Search planet for means to repair", 4.0, 1.0);
	CS.addText("TIP: Hold TAB for map, objectives and tips", 4.0, 1.0);
	CS.addWait("", function(time){
		if (pyramid1.getPosition().distance(ship.getPosition()) < pyramid1.getRadius() * 1.25)
		{
			canMove = false;
			makeSafe();
			return true;
		}
		return false;
	});
	CS.addPause(2.0);
	CS.addText("'Vessel, identify yourself'", 4.0, 0.5, {color:'green'});
	CS.addText("'Please respond'", 4.0, 0.5, {color:'green'});
	CS.addText("'..looks like an old drone sent here ages ago, sir.'", 4.0, 0.5, {color:'green'});
	CS.addText("'Scans show major hull damage, shall we repair it?", 5.0, 0.5, {color:'green'});
	CS.addText("'Roger that.'", 3.0, 0.5, {color:'green'});
	CS.addWait("STATUS: Foreign interface activated", function(time){
		if (ship.health === ship.maxHealth)
		{
			return true;
		}
		ship.moveTowardsAndStop(pyramid1.getPosition(), 1200, 1200, 0.5);
		ship.rotateTowardsAndStop(pyramid1.getAngle()+Math.PI, 0.75, 0.75/2.0);
		return false;
	});
	CS.addText("STATUS: Hull strength at 100%", 3.0, 1.0);
	CS.addText("'We could use a drone like this, we weren't expecting any Zenni here.'", 6.0, 0.5, {color:'green'});
	CS.addText("'Hmm, this technology is out dated. We'll need to upgrade this ship.'", 6.0, 0.5, {color:'green'});
	CS.addText("'Very interesting... The Zenni use a similar power system...'", 4.0, 0.5, {color:'green'});
	CS.addText("'We can instruct it to harvest generation cells from Zenni ships'", 6.0, 0.5, {color:'green'});
	CS.addText("'Roger that. Programming instructions...'", 4.0, 0.5, {color:'green'});
	CS.addText("STATUS: New Objective received.", 4.0, 0.5, {flashing:true});
	
	var setSubObj2 = function() {
		CSSub.clear();
		CSSub.addCall(function(self){
			self.lastMoney = playerMoney;
		});
		CSSub.addWait("", function(time, self) {
			if (playerMoney > self.lastMoney && (pyramid1.getPosition().distance(ship.getPosition()) < pyramid1.getRadius() * 0.75))
			{
				return true;
			}
			return false;
		});
		CSSub.addCall(function(self){
			if (playerMoney < 10)
			{
				save();
				self.setFlag("again");
				self.addText("'Need more cells than that, come back later.'", 4.0, 0.5, {color: 'green'});
			}
			else
			{
				canMove = false;
				self.setFlag("next");
				self.addText("'Excellent work. This drone really does well, sir.'",  4.0, 0.5, {color: 'green'});
				self.addText("STATUS: Objective Completed", 4.0, 0.5, {flashing:true});
			}
		});
	};
	
	CS.addCall(function(){
		zoneActive(zonesByTag("intro_outer"), 1);
		canMove = true;
		curObjective = "/MISSION1/LEVELUP";
		setSubObj2();
		save();
	});
	CS.addLoadPoint("/MISSION1/LEVELUP", function(){
		setSubObj2();
	});
	CS.addCall(function(){
		objectives = [ "OBJECTIVE: Destroy Zenni ships and return here to install upgrades." ];
		tips = [ "TIP: Return to Pyramid ship to heal and save." ];
	});
	CS.addText("OBJECTIVE: Destroy Zenni ships and return here to install upgrades.", 8.0, 4.0);
	CS.addWait("", function(){
		CSSub.updateRender();
		if (CSSub.complete())
		{
			if (CSSub.getFlag() === "next")
			{
				return true;
			}
			else
			{
				setSubObj2();
			}
		}
		return false;
	});
	CS.addCall(function(){
		playerLevel += 1;
		playerMoney -= 10.0;
		ship.attackPower *= 1.25;
		ship.armour *= 1.25;
		ship.health = ship.maxHealth;
	});
	CS.addText("STATUS: Upgrade level increased to 2!", 3.0, 0.5);
	CS.addText("STATUS: Cannon power increased by 25%!", 3.0, 0.5);
	CS.addText("STATUS: Armour increased by 25%!", 3.0, 0.5);
	CS.addCall(function(){
		curObjective = "/MISSION1/BOSS";
		save();
	});
	CS.addLoadPoint("/MISSION1/BOSS");
	CS.addPause(1.5);
	CS.addText("WARNING: Large warp fissure opening above", 4.0, 0.25, {color: 'red', flashing: true});
	CS.addCall(function(){
		pyramid1.cloak();
		zoneExclusive(zonesByTag("obj1")[0], true);
	});
	CS.addText("'Engaging cloak!'", 2.0, 0.25, {color: 'green'});
	CS.addCall(function(self){
		self.boss = new Boss1(closestObjectByClass(pyramid1.getPosition(), "Waypoint").getPosition());
		addObject(self.boss);
		canMove = true;
	});
	CS.addWait("", function(time, self) {
		if (self.boss.health <= 0.0)
		{
			self.boss = null;
			return true;
		}
		return false;
	});
	CS.addPause(2.0);
	CS.addCall(function(){
		canMove = false;
	});
	CS.addText("Success!", 4.0, 2.0, {flashing:true});
	CS.addCall(function(){
		zoneExclusive(null, false);
		canMove = false;
		ship.health = ship.maxHealth;
		pyramid1.decloak();
	});
	CS.addText("'My stars, the drone protected us without being programmed too!'", 6.0, 0.5, {color:'green'});
	CS.addText("'Captain: Quite strange... But probably just a glitch'", 6.0, 0.5, {color:'green'});
	CS.addText("'Captain: Another one of our ships may be on a nearby planet'", 6.0, 0.5, {color:'green'});
	CS.addText("'Captain: Instruct the drone to go there immediately.'", 6.0, 0.5, {color:'green'});
	CS.addText("'Roger that. Programming instructions...'", 4.0, 0.5, {color:'green'});
	CS.addCall(function(){
		curObjective = "/MISSION2/TRAVEL";
		save();
	});
	CS.addLoadPoint("/MISSION2/TRAVEL");
	CS.addCall(function(){
		canMove = true;
		zoneActive(zonesByTag("world2"), 1);
		zoneActive(zonesByTag("w2surface"), 1);
		zoneActive(zonesByTag("safe", zonesByTag("world2")), 1);
		zoneActive(zonesByTag("home"), 1);
		objectives = [ "OBJECTIVE: Locate and pass message to ship on nearby world" ];
		tips = [ "TIP: Upgrades are now automatic." ];
	});
	CS.addText("OBJECTIVE: Locate and pass message to ship on nearby world", 8.0, 4.0);
	
	CS.gotoLoadPoint(curObjective);
	//CS.gotoLoadPoint("_skip_intro");
	//ship.health = ship.maxHealth;
};
if (IN_EDITOR === false) init();

var _status = "";

var pcooldown = 0.0;

var cltick = 0;
ebind("update", global_event, function(e){
	pollKeyboard();
	
	if (canMove === true)
	{
		if (KEY.LEFT || KEY.A) ship.turnLeft();
		if (KEY.RIGHT || KEY.D) ship.turnRight();
		if (KEY.UP || KEY.W) ship.moveForward();
		if (KEY.DOWN || KEY.S) ship.moveBackward();
	}
	
	var zc = inZone(ship.getPosition());
	
	checkBattles();
	
	if (KEY.TAB)
		map_showing = Math.min(1.0, map_showing+8.0/FPS);
	else
		map_showing = Math.max(0.0, map_showing-4.0/FPS);
	
	if (shooting && canShoot)
	{
		if (pcooldown <= 0.0)
		{
			fireProjectile(ship, 'PlasmaCharge', getCam().fromScr(lastmp), {powerf: ship.attackPower});
			pcooldown = 0.1;
		}
		else
			pcooldown -= 1.0/FPS;
		//lazers.add(ship, getCam().fromScr(lastmp), 300/getCam().z);
	}
		
	cltick += 1;
	if (cltick === Math.floor(FPS/16))
	{
		if (clrLevel < curClrLevel)
			setColorLevel(--curClrLevel);
		else if (clrLevel > curClrLevel)
			setColorLevel(++curClrLevel);
		cltick = 0;
	}
		
	sunParks();
	
	if (canMove === false) shooting = false;
	
	//clrLevel = Math.floor(ship.health/ship.maxHealth*7.1);
	//clrLevel = Math.max(3, clrLevel);

	var c = getCam();
	c.p = ship.getPosition();
	setCam(c);
});

var BATTLE_MIN_DIST = 3500;
var BATTLE_MAX_DIST = 6000;

var in_battle = false;
var battle = {};
var b_lzone = null;
var b_lv = null;
var b_dist = BATTLE_MAX_DIST; // = BATTLE_MAX_DIST;

function checkBattles()
{
	if (in_battle === false)
	{
		var z = inZone(ship.getPosition());
		if (z && battles[z.tag])
		{
			b_dist -= (ship.getVelocity().length()*getCam().z)/FPS;
			if (b_dist < 0.0)
			{
				in_battle = true;
				b_dist = Math.random()*(BATTLE_MAX_DIST-BATTLE_MIN_DIST)+BATTLE_MIN_DIST;
				var ch = Math.random();
				var btl = battles[z.tag];
				var prbs = 0.0;
				var en = null;
				for (var i=0; i<btl.length; i++)
				{
					prbs += btl[i].weight;
					if (prbs >= ch)
					{
						en = btl[i].enemies;
						break;
					}
				}
				var oe = [];
				for (var i=0; i<en.length; i++)
				{
					var e = eval(en[i].prototype.className);
					e = new e(new pnt(0,0));
					e.p = null;
					for (var j=0; j<100; j++)
					{
						var r = Math.random() * YRES / 4 + YRES / 4;
						var a = Math.random() * Math.PI * 2.0;
						e.p = (new pnt(XRES/2, YRES/2)).add(pntFromAngle(a).mulf(r));
						e.p = getCam().fromScr(e.p);
						
						var z2 = inZone(e.p, e.r);
						if (z2 && z2.active === 1)
						{
							var ang = Math.abs(e.p.sub(ship.getPosition()).toAngle());
							var adiff = Math.abs(angleDiff(ang, ship.getAngle()-Math.PI/2.0));
							var planet = 0;
							var ob = closestObject(e.p);
							var obd = e.getActualRadius() + ob.getActualRadius();
							if (z.p.distance(e.p) < (z.r-e.r) && closestPlanet(e.p).distanceTo(e.p) > e.r * 1.5 && ob.p.distance(e.p) > obd && adiff < Math.PI / 2.5 && adiff > Math.PI/6.0)
							{
								break;
							}
							else
								e.p = null;
						}
						else
							e.p = null;
					}
					if (e.p !== null)
					{
						addObject(e);
						oe.push(e);
					}
				}
				battle.enemies = oe;
			}
		}
		b_lzone = z;
		_status = "" + Math.floor(b_dist);
	}
	else
	{
		var acount = battle.enemies.length;
		for (var i=0; i<battle.enemies.length; i++)
		{
			if (battle.enemies[i].onScreen(200) === false)
				removeObject(battle.enemies[i]);
			if (battle.enemies[i].id === -1)
				acount -= 1;
		}
		if (acount === 0)
		{
			battle = {};
			in_battle = false;
		}
	}
}

ebind("mouseup", global_event, function(e){
	if (e.b === 1)
		shooting = false;
});
ebind("mousedown", global_event, function(e){
	if (e.b === 1 && canMove === true)
		shooting = true;
});
ebind("mouseleave", global_event, function(e){
	shooting = false;
});
ebind("mousemove", global_event, function(e){
	lastmp = e.p.clone();
});

ebind("render_first", global_event, function(e){
	for (var i=0; i<stars.length; i++)
		stars[i].updateRender();
});

ebind("any_collision", global_event, function(e){
	var txta = 'static', txtb = 'static';
	if (e.obja !== 'static')
		txta = e.obja.className;
	if (e.objb !== 'static')
		txtb = e.objb.className;
	_status = txta + " vs " + txtb + " (" + Math.floor(e.im.length()) + ")";
});

ebind("render_last", global_event, function(e){

	var p = closestPlanet(ship.getPosition());
	var pd = p.p.distance(ship.getPosition());
	if (pd < p.r * 2.0 && p.distanceTo(ship.getPosition()) > 600.0/getCam().z  && getCam().z > 1.0)
	{
		var dv = p.p.sub(ship.getPosition());
		var off = getCam().toScr(ship.getPosition().add(dv.norm().mulf(64.0/getCam().z)));
		drawImageAng(loadImage('game/graphics/greenarrow.png'), off.x, off.y, 64.0, 0.5, dv.toAngle()+Math.PI/2.0);
	}

	drawZones();
	
	//drawString(XRES-stringWidth(_status, 16)-6, 6, _status, rgb(1,1,1), 16, rgb(.5,.5,.5));

	if (map_showing > 0.0)
	{
		var al = map_showing;
		var z = (YRES*0.5)/SYSTEM_RADIUS;
		var c = new pnt(0,0);//getCam().p;
		
		var ang = 0.0;
		for (var i=0; i<=3; i++)
		{
			drawLine(XRES/2-c.x*z, YRES/2-c.y*z, XRES/2-c.x*z + Math.cos(ang) * XRES, YRES/2-c.y*z + Math.sin(ang) * XRES, rgba(1,1,1,.45*al));
			drawLine(XRES/2-c.x*z, YRES/2-c.y*z, XRES/2-c.x*z - Math.cos(ang) * XRES, YRES/2-c.y*z - Math.sin(ang) * XRES, rgba(1,1,1,.45*al));
			ang += Math.PI/4.0;
		}
		
		drawCircleF(XRES/2-c.x*z, YRES/2-c.y*z, SYSTEM_RADIUS*z, rgba(0.0,0.0,0.0,.75*al));	
		drawCircleFG(XRES/2-c.x*z, YRES/2-c.y*z, SYSTEM_RADIUS*z-1.0, rgba(0.0,0.0,0.0,.75*al), SYSTEM_RADIUS*z+15.0, rgba(0.0,0.0,0.0,0.0));	
		
		for (var id in __planet)
		{
			var p = __planet[id];
			var zn = inZone(p.p);
			if (zn && zn.active === 1)
				drawCircleFG((p.p.x-c.x)*z+XRES/2, (p.p.y-c.y)*z+YRES/2, 0, rgba(0.7, 0.7, 0.7, 0.5*al), (p.r+300.0)*z, rgba(0.35, 0.35, 0.7, 0.25*al));
		}
		for (var id in __planet)
		{
			var p = __planet[id];
			var zn = inZone(p.p);
			if (zn && zn.active === 1)
				drawCircleFG((p.p.x-c.x)*z+XRES/2, (p.p.y-c.y)*z+YRES/2, 0, rgba(0.7, 0.7, 0.7, 0.5*al), p.r*z, rgba(0.3, 0.7, 0.3, 0.5*al));
		}
		var zn = inZone(new pnt(0,0));
		if (zn && zn.active === 1)
			drawCircleFG(XRES/2-c.x*z, YRES/2-c.y*z, 0, rgba(1, 0, 0, 0.5*al), SUN_RADIUS*z, rgba(1, 0, 0, .25*al));
		for (var id in __zone)
		{
			var zn = __zone[id];
			if (zn.active === 1 || (zn.parent && zn.parent.active===1))
			{
				if (zn.active === 0 || zn.parent === null || zn.parent.active === 0)
				{
					var clr;
					if (zn.active === 0 || (zn.parent && zn.parent.active===0))
						clr = rgba(.25,.25,1,0.5*al);
					else
						clr = rgba(.25,1,.25,0.5*al)
					drawCircle((zn.p.x-c.x)*z+XRES/2, (zn.p.y-c.y)*z+YRES/2, zn.r*z, clr );
				}
			}
		}
		var p = ship.getPosition();
		var a = ship.getAngle();
		drawImageAng(loadImage('game/graphics/ship.png'), XRES/2+p.x*z, YRES/2+p.y*z, 10, 1.0*al, a);
		
		var h = 12;
		var th = h * (objectives.length+tips.length);
		var x = 20, y = YRES-th-20.0;
		drawBoxF(x-5.0, y-2.5, XRES-x+5.0, y+th+5.0, rgba(0,0,0,al*0.5));
		for (var i=0; i<objectives.length; i++)
		{
			drawString(x, y, objectives[i], rgba(1,1,1,al*0.5), h, rgba(.5,.5,.5,al*0.5));
			y += h;
		}
		for (var i=0; i<tips.length; i++)
		{
			drawString(x, y, tips[i], rgba(1,1,1,al*0.5), h, rgba(.5,.5,.5,al*0.5));
			y += h;
		}
		
		/*
		// Random rendering tests
		drawBoxF(10, 10, 100, 100, rgba(0, 0, 0.5, 0.5));
		drawBox(10, 10, 100, 100, rgba(0.5, 0.5, 0.5, 0.5));
		drawPolygon(ship.getPosition(), 50.0, ctime(),
			[
				rgba(1,1,1,1),		pntFromAngle(0),
				rgba(1,0,0,0.5),	pntFromAngle(Math.PI/2.0),
				rgba(0,1,1,0.75),	pntFromAngle(Math.PI),
				rgba(0,0,0,0),		pntFromAngle(Math.PI*3.0/2.0)
			], getCam(), getCam().z);
		*/
	}
});

ebind("render_last", global_event, function(e){	
	var al = Math.min(0.5, (Math.abs(ship.heat)/(ship.health+5.0))*2.0);
	if (ship.health > 0.0)
	{
		if (ship.heat < 0.0)
		{
			drawBoxF(0, 0, XRES, YRES, rgba(0.0, 0.7, 0.7, al));
			var sp = getCam().toScr(ship.getPosition());
			drawImageAng(loadImage('game/graphics/healorb.png'), sp.x, sp.y, ship.getRadius()*1.35, al, 0.0);
		}
		else
			drawBoxF(0, 0, XRES, YRES, rgba(0.7, 0, 0, al));
	}
	else if (ship.health === 0.0)
		drawBoxF(0, 0, XRES, YRES, rgba(0.7, 0, 0, 1.0));	
	
	if (canMove === true)
	{
		drawImage(loadImage("game/graphics/hud-center.png"), XRES/2-62*0.5, 0.0, 62, 1.0);
		var mh = ship.maxHealth + 1;
		mh = Math.max(1.0, Math.round(mh));
		drawBoxF(XRES/2 - mh, 72 - 11 - 10, XRES/2 + mh, 72 - 10, rgba(.2,.6,.2,.5));
		var h = ship.health * 2;
		drawBoxF(XRES/2 - mh + 1, 72 - 11 - 10, XRES/2 - mh + 1 + h, 72 - 10, rgba(.2,1,.2,.5));
		
		if (playerMoney !== null)
		{
			var mtext = playerMoney + "";
			drawImage(loadImage('game/graphics/cell1.png'), XRES-32-24, 8.0, 48.0, 1.0);
			drawString(XRES-32-14-stringWidth(mtext, 24), 16, mtext, rgb(0, 1, 1), 24, rgb(0, .5, .5));
		}
	}
	
	var stime = ctime()-(lastSave+3.0);
	if (stime < 1.0)
	{
		var a = 1.0-Math.max(0, stime/1.0);
		var text = "Game Saved.";
		drawString(XRES/2-stringWidth(text, 26)/2.0, 72, text, rgba(1,1,1, a), 26, rgba(.5,.5,.5, a));
	}
});

ebind("render_after_objects", global_event, function(e){
	lazers.updateRender();
});

var deathClock = -1.0;

ebind("render_dead_last", global_event, function(e){
	var ft = Math.min(1.0-(ctime()-loadTime)/5.0, 1.0);
	if (ft > 0.0)
		drawBoxF(0, 0, XRES, YRES, rgba(0,0,0,ft));
	if (ship.health > 0.0)
	{
		CS.updateRender();
		if (in_battle === false)
		{
			var next = playerLevel + 1;
			if (levelUps[next])
			{
				var lu = levelUps[next];
				if (lu.money <= playerMoney)
				{
					makeSafe();
					canMove = false;
					playerLevel = next;
					playerMoney -= lu.money;
					CSLup.clear();
					CSLup.addPause(0.25);
					CSLup.addText("STATUS: Upgrade level increased to " + playerLevel + "!", 3.0, 0.5, {flashing:true});
					if (lu.maxHealth > 0)
					{
						var omax = ship.maxHealth;
						ship.maxHealth *= (1.0+lu.maxHealth);
						ship.health += ship.maxHealth - omax;
						CSLup.addText("STATUS: Maximum hitpoints increased by " + Math.floor(lu.maxHealth*100.0) + "%!", 3.0, 0.5);
					}
					if (lu.armour > 0)
					{
						ship.armour *= (1.0+lu.armour);
						CSLup.addText("STATUS: Armour increased by " + Math.floor(lu.armour*100.0) + "%!", 3.0, 0.5);
					}
					if (lu.attackPower > 0)
					{
						ship.armour *= (1.0+lu.attackPower);
						CSLup.addText("STATUS: Cannon power increased by " + Math.floor(lu.attackPower*100.0) + "%!", 3.0, 0.5);
					}
					CSLup.addCall(function(){canMove=true;});
				}
			}
		}
		CSLup.updateRender();
	}
	else
	{
		if (deathClock < 0.0)
			deathClock = ctime()+2.0;
		var t = (ctime()-deathClock)/3.0;
		t = Math.max(0.0, t);
		t = Math.min(1.0, t);
		var text = "You have been Annihilated.";
		drawBoxF(0, 0, XRES, YRES, rgba(0,0,0,t));
		drawString(XRES/2-stringWidth(text, 26)/2.0, YRES/2-26/2, text, rgba(1,1,1,t), 26, rgba(.5,0,0,t));
	}
});