registerObjectType('Boss1', 'Enemy');
var Boss1 = Enemy.extend({
    className: 'Boss1',
	isFixedZ: true,
	
	u: 		0.5,
	mass: 	10.0,
	r: 		110.0,
	
	health: 1500.0,
	worth:  20,
	
	isBoss: true,
	expTimer: 0.0,
	
	drones: null,
	droneTimer: 3.0,
	droneCount: 1,
	
	attackTimer: null,
	
	shootSpeed: 0.5,
	fireTicker: 0.5,
	attackRange: XRES,
	
    init: function () // Modifyable
    {
        arguments.callee.$.init.call(this); // Call super
		ebind("collision", this, this.collision, this);
		var pl = closestPlanet(this.getPosition());
		var dir = pl.p.sub(this.getPosition()).norm();
        this.setAngle(dir.toAngle()-Math.PI/2.0);
		this.startPos = this.getPosition();
		this.drones = [];
		
		this.attackTimer = new RotatingTimer([
			[ "wait",    2.5 ],
			[ "attack",  this.shootSpeed*4.0 ],
			[ "wait",    2.5 ],
			[ "attack",  this.shootSpeed*4.0 ],
		]);
    },
	collision: function(e)
	{
		if (e.obj && e.obj.className && e.obj.className === "PShip1")
		{
			e.damage(e.im.length());
		}
	},
	damage: function(d)
	{
		this.health -= d;
		if (this.health < 0)
		{
			for (var i=0; i<this.drones.length; i++)
				if (this.drones[i].health > 0)
					this.drones[i].damage(this.drones[i].health*2.0);
			this.health = 0;
			explosion(this.getPosition(), this.getActualRadius(), [rgb(1,.25,0,1), rgb(1,.5,0), rgb(1,.5,0)], this.isBoss);
			
			this.addCells();
			
			removeObject(this);
		}
	},
    editorRender: function (c) // Modifyable - Only function called by editor
    {
		var pg = this.getPosition();
        var r = this.getRadius();
        var p = c.toScr(pg);
		if (this.isFixedZ === false || IN_EDITOR === true) r *= c.z;
		var fade = Math.sin(ctime()*Math.PI)*0.5+0.5;
        drawImageAng(loadImage('game/graphics/boss1-1.png'), p.x, p.y, r*1.5, 1.0*this.warpinOpacity(), this.getAngle());
		drawImageAng(loadImage('game/graphics/boss1-2.png'), p.x, p.y, r*1.5, fade*this.warpinOpacity(), this.getAngle());
    },
    gameRender: function (c) // Modifyable
    {
		this.renderWarpin();
        this.editorRender(c);
		//this.renderHealthBar(c);
    },
    update: function ( dt ) // Modifyable
    {
		var c = getCam();
		this.moveTowardsAndStop(this.startPos, 400*c.z, 1200*c.z, 0.5);
		if (this.health < 500.0)
		{
			this.expTimer += 1.0/FPS;
			if (this.expTimer > (1.5-(1.45-this.health/500.0)))
			{
				explosion(this.getPosition().add(pntFromAngle(Math.random()*Math.PI).mulf(this.getActualRadius()/1.5)), this.getActualRadius()/4.0, [rgb(1,.25,0,1), rgb(1,.5,0), rgb(1,.5,0)]);
				this.expTimer = 0.0;
			}
		}
		var dc = this.droneCount;
		if (this.health < 1000.0)
			dc += 1;
		if (this.health < 500.0)
			dc += 1;
		if (dc > this.drones.length)
		{
			this.droneTimer -= 1.0/FPS;
			if (this.droneTimer < 0.0)
			{
				for (var i=this.drones.length; i<dc; i++)
				{
					var a = (i / dc) * Math.PI * 2.0;
					this.drones[i] = new ADrone(this.getPosition().add(pntFromAngle(a).mulf(this.getRadius()/c.z*1.5)));
					this.drones[i].health /= 2.0;
					this.drones[i].speedf = 0.5;
					this.drones[i].worth = 0.0;
					this.drones[i].attackPower *= 1.5;
					addObject(this.drones[i]);
				}
				this.droneTimer = 3.0;
			}
		}
		else
		{
			for (var i=0; i<this.drones.length; i++)
				if (this.drones[i].health === 0.0)
				{
					this.drones.splice(i,1);
					i--;
				}
		}
		
		var attackState = this.attackTimer.update();
		
		this.fireTicker -= 1.0/FPS;
		var shooting = false;
		if (this.fireTicker < 0.0)
		{
			this.fireTicker = this.shootSpeed;
			shooting = true;
		}
		
		var dist = ship.getPosition().distance(this.getPosition());
		if (this.canSee(ship) && dist <= this.attackRange/c.z)
		{
			if (attackState === "attack" && shooting)
			{
				var miss = calcMiss(dist, 50/c.z, 600/c.z, 40/c.z, 100/c.z);
				fireProjectile(this, 'PlasmaCharge', ship.getPosition().add(miss), {powerf: 4.0, speedF: 0.25, radiusF: 2.5});
			}
		}		
		
	},
	getCollisionObj: function ()
	{
		return this.collisionObjectImage('game/graphics/boss1-1.png');
	},
});


