registerObjectType('ADrone', 'Enemy');
var ADrone = Enemy.extend({
    className: 'ADrone',
	isFixedZ: true,
	
	u: 		0.5,
	mass: 	2.0,
	r: 		17.5,
	
	health: 60.0,
	worth:  1,
	attackPower: 1.5,
	speedf: 1.0,
	
	attackRange: 		500,
	attackTrackDelay: 	0.5,
	attackTracker: 		null,
	attackTimer: 		null,
	rotateTimer: 		null,
	
	shootSpeed: 0.25,
	fireTicker: 1.0,
    
    init: function () // Modifyable
    {
        arguments.callee.$.init.call(this); // Call super
		ebind("collision", this, this.collision, this);
		this.attackTracker = new DistanceDelayedTracker(null);
		this.attackTimer = new RotatingTimer([
			[ "wait",    0.5 ],
			[ "attack",  0.5 ],
		]);
		this.rotateTimer = new RotatingTimer([
			[  1, 3.0 ],
			[ -1, 3.0 ]
		]);
    },
	collision: function(e)
	{
		if (e.obj && e.obj.className && e.obj.className === "PShip1")
		{
			//this.setVelocity(e.im.mulf(-5.0));
		}
	},
	damage: function(d)
	{
		if (this.health >= 7.5 && (this.health-d) < 7.5)
		{
			this.attackTimer = new RotatingTimer([
				[ "wait", 0.75 ],
				[ "attack", 0.25 ],
			]);
		}
		this.health -= d;
		if (this.health < 0)
		{
			this.health = 0;
			explosion(this.getPosition(), this.getActualRadius(), [rgb(1,.25,0,1), rgb(1,.5,0), rgb(1,.5,0)]);
			
			this.addCells();
			
			removeObject(this);
		}
	},
    editorRender: function (c) // Modifyable - Only function called by editor
    {
		var pg = this.getPosition();
        var r = this.getRadius();
        var p = c.toScr(pg);
		if (this.isFixedZ === false || IN_EDITOR === true) r *= c.z;
		var fade = Math.sin(ctime()*30.0)*0.5+0.5;
        drawImageAng(loadImage('game/graphics/adrone-1.png'), p.x, p.y, r*1.5, 1.0*this.warpinOpacity(), this.getAngle());
		drawImageAng(loadImage('game/graphics/adrone-2.png'), p.x, p.y, r*1.5, fade*this.warpinOpacity(), this.getAngle());
    },
    gameRender: function (c) // Modifyable
    {
		this.renderWarpin();
        this.editorRender(c);
		this.renderHealthBar(c);
    },
    update: function ( dt ) // Modifyable
    {
		var c = getCam();
		
		// Track player
		var rotd = this.rotateTimer.update();
		this.trackObject(ship, 0.25, (XRES/1.5)/c.z, this.getActualRadius()+ship.getActualRadius()+150/c.z, 300.0*this.speedf, 600.0*this.speedf, 0.25, 30.0*rotd);
		
		// Avoid allies
		var ally = closestObjectByCGroup(this.p, this.cgroup, this);
		if (ally && ally.isProjectile !== true)
			this.avoidObject(ally, (this.getRadius()+ally.getRadius()+50)/c.z, 125.0*this.speedf, 25.0*this.speedf);

		// Avoid Player & Projectiles
		var ob = closestObjectByCGroup(this.p, 1);
		if (ob)
			this.avoidObject(ob, (ob.getRadius()+this.getRadius()+50)/c.z, 125.0*this.speedf, 25.0*this.speedf);
		this.avoidObject(ship, (ship.getRadius()+this.getRadius()+150)/c.z, 250.0*this.speedf, 50.0*this.speedf);
		
		this.rotateTowardsAndStop(0.0, 0.25);
		
		// Shooting
		var attackState = this.attackTimer.update();
		
		this.fireTicker -= 1.0/FPS;
		var shooting = false;
		if (this.fireTicker < 0.0)
		{
			this.fireTicker = this.shootSpeed;
			shooting = true;
		}
		
		var dist = ship.getPosition().distance(this.getPosition());
		if (this.canSee(ship) && dist <= this.attackRange/c.z)
		{
			if (attackState === "attack" && shooting)
			{
				//var miss = calcMiss(dist, 50/c.z, 300/c.z, 30/c.z, 80/c.z);
				var miss = new pnt(0,0);
				fireProjectile(this, 'PlasmaCharge', ship.getPosition().add(miss), {powerf: this.attackPower, /*project:ship.getVelocity(), projectR: ship.getRadius()/c.z, projectMissMax: Math.PI/8.0,*/ speedF: 0.5});
			}
		}		
	},
	getCollisionObj: function ()
	{
		return this.collisionObjectImage('game/graphics/adrone-1.png');
	},
});

