registerObjectType('Projectile', 'Dynamic');
var Projectile = Dynamic.extend({
    className: 'Projectile',
	isProjectile: true,
	health: 1.0,
    
    init: function () // Modifyable
    {
		ebind("collision", this, this.collision, this);
		arguments.callee.$.init.call(this); // Call super
    },
	damage: function(d)
	{
		this.health -= d;
		if (this.health < 0)
		{
			this.health = 0;
			explosion(this.getPosition(), this.getActualRadius()*1.5, [rgb(1,.25,0,1), rgb(1,.5,0), rgb(1,.5,0)]);			
			removeObject(this);
		}
	},
	collision: function(e)
	{
		if (e.obj && e.obj.className)
		{
			if (e.obj.damage)
				e.obj.damage(e.im.length()*this.getRadius()*getCam().z);
			if (e.obj.isProjectile !== true || e.obj.health > this.health)
				this.damage(this.health*2.0);
		}
		else
			this.damage(this.health*2.0);
	},
    editorRender: function (c) // Modifyable - Only function called by editor
    {
        var p = c.toScr(this.getPosition());
        var r = this.getRadius();
        r *= c.z;
        drawImage(loadImage('gfx/mob.png'), p.x-r, p.y-r, r*2.0);
    },
    gameRender: function (c) // Modifyable
    {
        var p = c.toScr(this.getPosition());
        var r = this.getRadius();
        if (!this.isFixedZ) r *= c.z;
        drawImage(loadImage('gfx/mob.png'), p.x-r, p.y-r, r*2.0);
    },
    update: function ( dt ) // Modifyable
    {
		if (this.onScreen(100.0) === false)
			removeObject(this);
    },
    getCollisionObj: function () // Modifyable
    {
        return this.collisionObjectCircle();
    },
	fireTowards: function(towards, startV, opts)
	{
		var diff = towards.sub(this.getPosition());
		var z = getCam().z;
		var sp = this.speed/z*(opts.speedF?opts.speedF:1.0);
		var dist = diff.length;
		var vel = diff.norm().mulf(sp);
		if (opts.radiusF)
			this.setRadius(this.getRadius()*opts.radiusF);
		var rthis = this.getRadius()/z;
		if (opts.project)
		{
			var vel2 = opts.project;
			var test = function(a)
			{
				var v2 = pntFromAngle(a).mulf(sp);
				return ccvvTimeIntersect(diff, new pnt(0,0), vel2, v2, opts.projectR, rthis);
			};
			var as = (Math.PI*2.0)/500.0;
			var mt = null;
			var bang = null;
			for (var a=0.0; a<Math.PI*2.0; a+=as)
			{
				var t = test(a);
				if (t !== null)
				{
					if (mt === null || (t < mt))
					{
						mt = t;
						bang = a;
					}
				}
			}
			if (bang !== null)
			{
				if (opts.projectMissMax)
				{
					bang = bang + opts.projectMissMax*2.0*Math.random() - opts.projectMissMax;
				}
				vel = pntFromAngle(bang).mulf(sp);
			}
		}
		this.setVelocity(vel.add(startV));
		if (this.cgroup === 2)
		{
			this.setVelocity(this.getVelocity().sub(startV));
			this.clr = rgb(1,0,0);
		}
	},
});

